/*
 * $Id: BasicMessage.java 366395 2006-01-06 02:42:19Z niallp $
 * $Revision: 366395 $
 * $Date: 2006-01-05 21:42:19 -0500 (Thu, 05 Jan 2006) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.io.Serializable;

import org.apache.commons.resources.Message;

/**
 * A basic implementation of the Message interface.
 * <p>
 * Orginally based on org.apache.struts.action.ActionMessage, Revision 49176.
 */
public class BasicMessage implements Serializable, Message {

    /**
     * The logical name of the {@link org.apache.commons.resources.Resources} 
     * instance this message is associated with (optional).
     */
    private String resourcesName;
    
    /**
     * The message key for this message.
     */
    private String key = null;

    /**
     * The replacement values for this message.
     */
    private Object[] values = null;

    /**
     * Default Constructor.
     */
    public BasicMessage() {
        super();
    }
    
    /**
     * Construct a message with no replacement values.
     *
     * @param key Message key for this message
     */
    public BasicMessage(String key) {

        this.key = key;
        this.values = null;

    }

    /**
     * Construct a message with the specified replacement values.
     *
     * @param key Message key for this message
     * @param value0 First replacement value
     */
    public BasicMessage(String key, Object value0) {

        this.key = key;
        this.values = new Object[]{value0};

    }

    /**
     * Construct a message with the specified replacement values.
     *
     * @param key Message key for this message
     * @param value0 First replacement value
     * @param value1 Second replacement value
     */
    public BasicMessage(String key, Object value0, Object value1) {

        this.key = key;
        this.values = new Object[]{value0, value1};

    }

    /**
     * Construct a message with the specified replacement values.
     *
     * @param key Message key for this message
     * @param value0 First replacement value
     * @param value1 Second replacement value
     * @param value2 Third replacement value
     */
    public BasicMessage(String key, Object value0, Object value1,
                        Object value2) {

        this.key = key;
        this.values = new Object[]{value0, value1, value2};

    }

    /**
     * Construct a message with the specified replacement values.
     *
     * @param key Message key for this message
     * @param value0 First replacement value
     * @param value1 Second replacement value
     * @param value2 Third replacement value
     * @param value3 Fourth replacement value
     */
    public BasicMessage(String key, Object value0, Object value1,
                        Object value2, Object value3) {

        this.key = key;
        this.values = new Object[]{value0, value1, value2, value3};
    }

    /**
     * Construct a message with the specified replacement values.
     *
     * @param key Message key for this message
     * @param values Array of replacement values
     */
    public BasicMessage(String key, Object[] values) {
        this.key = key;
        this.values = values;
    }

    /**
     * <p>Return the logical name of the {@link org.apache.commons.resources.Resources} 
     * instance this message is associated with.</p>
     *
     * @return The name of the resources instance.
     */
    public String getResourcesName() {
        return resourcesName;
    }

    /**
     * <p>Set the logical name of the {@link org.apache.commons.resources.Resources} 
     * instance this message is associated with.</p>
     *
     * @param resourcesName The name of the resources instance.
     */
    public void setResourcesName(String resourcesName) {
        this.resourcesName = resourcesName;
    }

    /**
     * @return Get the message key for this message.
     */
    public String getKey() {
        return (this.key);
    }
    
    /**
     * Set the key for the message.
     *
     * @param key The key to set.
     */
    public void setKey(String key) {
        this.key = key;
    }

    /**
     * @return Get the replacement values for this message.
     */
    public Object[] getValues() {
        return (this.values);
    }

    /**
     * @param values The replacement values for this message.
     */
    public void setValues(Object[] values) {
        this.values = values;
    }

    /**
     * Returns a String in the format: resourcesName:key[value0, value1, etc]. 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer buff = new StringBuffer();
        if (this.resourcesName != null) {
            buff.append(this.resourcesName);
            buff.append(":"); 
        }
        buff.append(this.key);
        buff.append("[");

        if (this.values != null) {

            for (int i = 0; i < this.values.length; i++) {

                buff.append(this.values[i]);

                // don't append comma to last entry
                if (i < this.values.length - 1) {
                    buff.append(", ");
                }
                
            }
        }

        buff.append("]");

        return buff.toString();
    }

}
