/*
 * $Id: BasicMessageList.java 366395 2006-01-06 02:42:19Z niallp $
 * $Revision: 366395 $
 * $Date: 2006-01-05 21:42:19 -0500 (Thu, 05 Jan 2006) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.resources.Message;
import org.apache.commons.resources.MessageList;

/**
 * A basic implementation of a MessageList.
 * <p>
 * Orginally based on org.apache.struts.action.ActionMessages, Revision 49670.
 */
public class BasicMessageList implements Serializable, MessageList {

    /**
     * Compares MessageItems.
     */
    private static final Comparator ACTION_ITEM_COMPARATOR = new Comparator() {
        public int compare(Object o1, Object o2) {
            return ((MessageItem) o1).getOrder() - ((MessageItem) o2).getOrder();
        }
    };
    
    /**
     * Have the messages been retrieved from this object?
     */
    private boolean accessed = false;

    /**
     * The accumulated set of <code>Message</code> objects (represented
     * as a List) for each property, keyed by property name.
     */
    private Map messages = new HashMap();

    /**
     * The current number of the property/key being added.  This is used
     * to maintain the order messages are added.
     */
    private int count = 0;

    // --------------------------------------------------------- Public Methods

    /**
     * Create an empty <code>MessageList</code> object.
     */
    public BasicMessageList() {
        super();
    }

    /**
     * Create an <code>MessageList</code> object initialized to use
     * the given value for the "global" message key.
     *
     * @param globalMessageKey The new default global message key
     */
    public BasicMessageList(String globalMessageKey) {
        super();
        this.setGlobalMessageKey(globalMessageKey);
    }

    /**
     * Create an <code>MessageList</code> object initialized with the given
     * messages.
     *
     * @param messages The messages to be initially added to this object.
     */
    public BasicMessageList(MessageList messages) {
        super();
        this.add(messages);
    }

    /**
     * Create an <code>MessageList</code> object initialized with the given
     * messages and the given global message key.
     *
     * @param globalMessageKey The new default global message key
     * @param messages The messages to be initially added to this object.
     * 
     */
    public BasicMessageList(String globalMessageKey, MessageList messages) {
        super();
        this.setGlobalMessageKey(globalMessageKey);
        this.add(messages);
    }

    /**
     * The "global" message key for this MessageList
     * [GLOBAL_MESSAGE_KEY].
     */
    private String globalMessageKey = GLOBAL_MESSAGE_KEY;

    /**
     * @return The default global message key
     */
    public String getGlobalMessageKey() {
        return this.globalMessageKey;
    }

    /**
     * @param globalMessageKey The new default global message key
     */
    public void setGlobalMessageKey(String globalMessageKey) {
        this.globalMessageKey = globalMessageKey;
    }

    /**
     * Add a message to the set of messages for the specified property.  An
     * order of the property/key is maintained based on the initial addition
     * of the property/key.
     *
     * @param property  Property name (or MessageList.GLOBAL_MESSAGE_KEY)
     * @param message   The message to be added
     */
    public void add(String property, Message message) {

        MessageItem item = (MessageItem) messages.get(property);
        List list = null;

        if (item == null) {
            list = new ArrayList();
            item = new MessageItem(list, this.count++);

            messages.put(property, item);
        } else {
            list = item.getList();
        }

        list.add(message);

    }

    /**
     * Add a message to the set of messages for the "global" property.  An
     * order of the property/key is maintained based on the initial addition
     * of the property/key.
     *
     * @param message   The message to be added
     */
    public void add(Message message) {
        this.add(getGlobalMessageKey(), message);
    }

    /**
     * Adds the messages from the given <code>MessageList</code> object to
     * this set of messages. The messages are added in the order they are returned from
     * the properties() method. If a message's property is already in the current
     * <code>MessageList</code> object it is added to the end of the list for that
     * property. If a message's property is not in the current list it is added to the end
     * of the properties.
     *
     * @param messageList The <code>MessageList</code> object to be added.
     */
    public void add(MessageList messageList) {
        // loop over properties
        Iterator props = messageList.properties();
        while (props.hasNext()) {
            String property = (String) props.next();

            // loop over messages for each property
            Iterator msgs = messageList.get(property);
            while (msgs.hasNext()) {
                Message msg = (Message) msgs.next();
                this.add(property, msg);
            }

        }
    }

    /**
     * Clear all messages recorded by this object.
     */
    public void clear() {
        this.messages.clear();
    }
    
    /**
     * Determines if the MessageList's messages have been accessed one or more
     * times.  Returns <code>true</code> if the <code>get()</code> or 
     * <code>get(String)</code> methods are called.  
     * @return <code>true</code> if the messages have been accessed one or more
     * times.
     */
    public boolean isAccessed() {
        return this.accessed;
    }

    /**
     * @return Return <code>true</code> if there are no messages recorded
     * in this collection, or <code>false</code> otherwise.
     */
    public boolean isEmpty() {
        return messages.isEmpty();
    }

    /**
     * Return the set of all recorded messages, without distinction
     * by which property the messages are associated with.  If there are
     * no messages recorded, an empty enumeration is returned.
     *
     * @return All messages.
     */
    public Iterator get() {
        this.accessed = true;
        
        if (messages.isEmpty()) {
            return Collections.EMPTY_LIST.iterator();
        }

        ArrayList results = new ArrayList();
        ArrayList actionItems = new ArrayList();

        for (Iterator i = messages.values().iterator(); i.hasNext();) {
            actionItems.add(i.next());
        }

        // Sort MessageItems based on the initial order the
        // property/key was added to MessageList.
        Collections.sort(actionItems, ACTION_ITEM_COMPARATOR);

        for (Iterator i = actionItems.iterator(); i.hasNext();) {
            MessageItem ami = (MessageItem) i.next();

            for (Iterator msgs = ami.getList().iterator(); msgs.hasNext();) {
                results.add(msgs.next());
            }
        }

        return results.iterator();
    }

    /**
     * Return the set of messages related to a specific property.
     * If there are no such messages, an empty enumeration is returned.
     *
     * @param property Property name
     * @return Messages related to a specific property.
     */
    public Iterator get(String property) {
        this.accessed = true;
        
        MessageItem item = (MessageItem) messages.get(property);
        
        return (item == null)
            ? Collections.EMPTY_LIST.iterator()
            : item.getList().iterator();
    }

    /**
     * Return the set of property names for which at least one message has
     * been recorded.  If there are no messages, an empty Iterator is returned.
     * If you have recorded global messages, the String value of
     * <code>MessageList.GLOBAL_MESSAGE</code> will be one of the returned
     * property names.
     *
     * @return The property names.
     */
    public Iterator properties() {
        return messages.keySet().iterator();
    }

    /**
     * Return the number of messages recorded for all properties (including
     * global messages).  <strong>NOTE</strong> - it is more efficient to call
     * <code>isEmpty()</code> if all you care about is whether or not there are
     * any messages at all.
     *
     * @return The number of messages.
     */
    public int size() {

        int total = 0;

        for (Iterator i = messages.values().iterator(); i.hasNext();) {
            MessageItem ami = (MessageItem) i.next();
            total += ami.getList().size();
        }

        return total;
    }

    /**
     * Return the number of messages associated with the specified property.
     *
     * @param property Property name (or MessageList.GLOBAL_MESSAGE_KEY
     * @return The number of messages for a specific property.
     */
    public int size(String property) {

        MessageItem ami = (MessageItem) messages.get(property);

        return (ami == null) ? 0 : ami.getList().size();
    }
    
    /**
     * Returns a String representation of this MessageList's 
     * [property name]=[message list] mapping.
     * @see java.lang.Object#toString()
     */    
    public String toString() {
        return this.messages.toString();
    }

    /**
     * Holds messages for a specified property.
     */    
    protected static class MessageItem implements Serializable {

        /**
         * The list of <code>Message</code>s.
         */
        private List list = null;

        /**
         * The position in the list of messages.
         */
        private int order = 0;

        /**
         * Construct a MessageItem with a list of messages
         * and specified order.
         * @param list List of Messages.
         * @param order Order.
         */
        public MessageItem(List list, int order) {
            this.list = list;
            this.order = order;
        }

        /**
         * @return The list of messsages.
         */
        public List getList() {
            return list;
        }

        /**
         * @param list The list of messsages.
         */
        public void setList(List list) {
            this.list = list;
        }

        /**
         * @return The order.
         */
        public int getOrder() {
            return order;
        }

        /**
         * @param order The order.
         */
        public void setOrder(int order) {
            this.order = order;
        }

    }

}
