/*
 * $Id: ResourceBundleResources.java 358692 2005-12-23 03:37:26Z niallp $
 * $Revision: 358692 $
 * $Date: 2005-12-22 22:37:26 -0500 (Thu, 22 Dec 2005) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.util.Collections;
import java.util.Iterator;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.resources.ResourcesException;
import org.apache.commons.resources.ResourcesKeyException;
import org.apache.commons.resources.util.IteratorEnumeration;

/**
 * <p>Concrete implementation of 
 * {@link org.apache.commons.resources.Resources} that wraps a set
 * (one per Locale) of <code>java.util.ResourceBundle</code> instances
 * that share a common base name.</p>
 */
public class ResourceBundleResources extends ResourcesBase {

    /**
     * <p>The logging instance for this class.</p>
     */
    private transient Log log =
        LogFactory.getLog(ResourceBundleResources.class);

    // ----------------------------------------------------------- Constructors


    /**
     * <p>Create a new {@link org.apache.commons.resources.Resources} 
     * instance with the specified
     * logical name and bundle base name.</p>
     *
     * @param name Logical name of the new instance
     * @param base Fully qualified base name of the <code>ResourceBundle</code>
     *  instances to be wrapped
     */
    public ResourceBundleResources(String name, String base) {

        super(name);
        this.base = base;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The fully qualified base name of the <code>ResourceBundle</code>
     * instances to be wrapped.</p>
     */
    private String base = null;


    // ------------------------------------------------------------- Properties


    /**
     * <p>Return the fully qualified base name of the
     * <code>ResourceBundle</code> instances we are wrapping.</p>
     * @return The base name of this resources instance.
     */
    public String getBase() {

        return (this.base);

    }

    /**
     * <p>Return an <code>Iterator</code> over the defined keys in this
     * {@link org.apache.commons.resources.Resources} instance.</p>
     *
     * @return The keys contained in this resources instance.
     */
    public Iterator getKeys() {

        try {
            ResourceBundle bundle = getBundle(null);
            return new IteratorEnumeration(bundle.getKeys());
            
        } catch (MissingResourceException e) {
            return Collections.EMPTY_LIST.iterator();
        }

    }


    // ---------------------------------------------- Content Retrieval Methods


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * Object, localized based on the specified <code>locale</code>.
     * </p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @return content for a specified key.
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public Object getObject(String key, Locale locale) {

        if (getLog().isTraceEnabled()) {
            getLog().trace("Retrieving message for key '" + key + "' and locale '"
                      + locale + "'");
        }

        try {
            ResourceBundle bundle = getBundle(locale);
            Object object = bundle.getObject(key);
            if (getLog().isTraceEnabled()) {
                getLog().trace("Retrieved object for key '" + key + 
                               "' and locale '" + locale +
                               "' is '" + object + "'");
            }
            return object;
            
        } catch (MissingResourceException e) {
            if (getLog().isTraceEnabled()) {
                getLog().trace("No resource found for key '" + key + 
                               "' and locale '" + locale + "'");
            }
            if (isReturnNull()) {
                return (null);
            } else {
                throw new ResourcesKeyException(key);
            }
        }

    }


    // ------------------------------------------------------ Protected Methods


    /**
     * <p>Return the appropriate <code>ResourceBundle</code> instance
     * that corresponds to the specified <code>locale</code> parameter.
     * The first time a particular bundle is requested, cache it so
     * that subsequent requests will operate more quickly.</p>
     *
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @return The Resource Bundle corresponding to the locale and time zone.
     *
     * @exception MissingResourceException if the requested Resourcebundle
     *  cannot be acquired
     */
    protected ResourceBundle getBundle(Locale locale)
        throws MissingResourceException {

        if (locale == null) {
            locale = Locale.getDefault();
        }

        // Locate the appropriate ClassLoader
        ClassLoader loader = getClassLoader();

        // Locate the requested ResourceBundle instance
        ResourceBundle bundle = ResourceBundle.getBundle(base, locale, loader);

        return (bundle);

    }


    /**
     * <p>Return the <code>ClassLoader</code> for which we are mapping
     * <code>ResourceBundle</code> instances.</p>
     * @return The Class Loader for the resource bundle.
     */
    protected ClassLoader getClassLoader() {

        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        
        if (loader == null) {
            loader = this.getClass().getClassLoader();
        }
        
        return (loader);

    }

    /**
     * Accessor method for Log instance.
     *
     * The Log instance variable is transient and
     * accessing it through this method ensures it
     * is re-initialized when this instance is
     * de-serialized.
     *
     * @return The Log instance.
     */
    private Log getLog() {
        if (log == null) {
            log =  LogFactory.getLog(ResourceBundleResources.class);
        }
        return log;
    }

}
