/*
 * $Id: MessagesTestCase.java 354761 2005-12-07 15:11:58Z niallp $
 * $Revision: 354761 $
 * $Date: 2005-12-07 10:11:58 -0500 (Wed, 07 Dec 2005) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2005 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources;

import java.util.Locale;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.commons.resources.impl.ResourceBundleResourcesFactory;

/**
 * <p>Unit tests for
 * <code>org.apache.commons.resources.Messages</code>.
 * </p>
 */
public class MessagesTestCase extends TestCase {

    // Configuration value for our resource bundles
    protected static final String CONFIG = "org.apache.commons.resources.impl.TestResources";

    // Logical name of the Resources instance to be created
    protected static final String NAME = "name";


    // The ResourcesFactory to use for acquiring the Resources
    protected ResourcesFactory factory = null;


    // The Resources instance containing our messages
    protected Resources resources = null;

    // The Messages instance to be tested
    protected Messages messages = null;


    // ----------------------------------------------------------- Constructors


    public MessagesTestCase(String name) {
        super(name);
    }


    // --------------------------------------------------- Overall Test Methods


    // Set up instance variables required by this test case
    public void setUp() throws Exception {
        factory = new ResourceBundleResourcesFactory();
        resources = factory.getResources(NAME, CONFIG);
        resources.setReturnNull(false);
        messages = new Messages(resources);
    }

    // Return the tests included in this test suite
    public static Test suite() {
        return (new TestSuite(MessagesTestCase.class));
    }

    // Tear down the instance variables required by this test case
    public void tearDown() {
        messages = null;
        resources = null;
        factory = null;
    }


    // ------------------------------------------------ Individual Test Methods


    /**
     * Test the use of the shortcut getMessages() method.
     */
    public void testLocal() {
        String message = null;
        
        Messages local =
            Messages.getMessages(
                "org.apache.commons.resources.impl.LocalStrings");
        
        assertNotNull("Local messages found", local);

        message = local.getMessage("local.message");
        
        assertEquals(
            "Correct individual message",
            "[Local] REPLACE {0} WITH {1}",
            message);

        message =
            local.getMessage("local.message", new String[] { "abc", "def" });
            
        assertEquals(
            "Correct replaced message",
            "[Local] REPLACE abc WITH def",
            message);

        local.getResources().setReturnNull(false);
        message = local.getMessage("local.missing");
        assertEquals("Correct missing message", "???local.missing???", message);
        

        Messages messages2 = new MyMessages(null);
        Messages messages3 = messages2.getMessages(null);
        assertNull("Invalid Messages", messages3);

    }
    


    /**
     * Test the ability to retrieve messages with replacements.
     */
    public void testMessage() {
        String message = null;

        message = messages.getMessage("test.missing");
        assertEquals("Correct missing message", "???test.missing???", message);
        
        message = messages.getMessage(Locale.ENGLISH, "test.message");
        assertEquals(
            "Correct individual message",
            "[Base] REPLACE {0} WITH {1}",
            message);

        message =
            messages.getMessage("test.message", new String[] { "abc", "def" });
        
        assertEquals(
            "Correct replaced message",
            "[Base] REPLACE abc WITH def",
            message);

        message =
            messages.getMessage(Locale.ENGLISH, "test.message", new String[] { "abc", "def" });
        
        assertEquals(
            "Correct replaced message",
            "[Base] REPLACE abc WITH def",
            message);

        message =
            messages.getMessage(Locale.ENGLISH, "test.message.single", "abc");
        
        assertEquals(
            "Correct replaced message",
            "[Base] REPLACE abc",
            message);
        
        message =
            messages.getMessage("test.message.single", "abc");
        
        assertEquals(
            "Correct replaced message",
            "[Base] REPLACE abc",
            message);
        
        Resources resources2 = 
            factory.getResources(NAME + "-does-not-exist", 
                    CONFIG + "-does-not-exist");
        resources2.setReturnNull(false);
        Messages messages2 = new Messages(resources2);
        message = messages2.getMessage("test.message");
        
        Exception exception = new Exception("Bad Implementation");
        ResourcesException resourcesException = new ResourcesException(exception);
        try{
            if (true)
                 throw resourcesException;
        }
        catch(ResourcesException e){
            assertEquals("Check exception", e, resourcesException);
            assertEquals("Check exception cause", e.getRootCause(), exception);
            assertEquals("Check exception message", e.getMessage(), exception.getMessage());
        }
        
    }


    /**
     * Test the ability to retrieve messages with replacements.
     */
    public void testMessages() {
        String message = null;

        message = messages.getMessage("test.message");
        assertEquals(
            "Correct individual message",
            "[Base] REPLACE {0} WITH {1}",
            message);

        message =
            messages.getMessage("test.message", new String[] { "abc", "def" });
        
        assertEquals(
            "Correct replaced message",
            "[Base] REPLACE abc WITH def",
            message);

        message = messages.getMessage("test.missing");
        assertEquals("Correct missing message", "???test.missing???", message);
        
        
    }


    /**
     * Test the characteristics of a newly created instance.
     */
    public void testPristine() {
        assertNotNull(messages);
        assertTrue(
            "Correct wrapped resources",
            resources == messages.getResources());
    }


    class MyMessages extends Messages{

        public MyMessages(Resources resources) {
            super(resources);
            ResourcesFactory factory = new ResourcesFactory() {
                public Resources getResources(String name)
                                 throws ResourcesException {
                    throw new ResourcesException("Bad Implementation");
                }

                public boolean isReturnNull() {
                    return false;
                }

                public void setReturnNull(boolean returnNull) {}

                public Resources getResources(String name, String config) throws ResourcesException {
                    return null;
                }

                public void release() throws ResourcesException {
                }
            };
            Messages.setFactory(factory);
        }
        
    }
}
