/*
 * $Id: JDBCResourcesTestCase.java 354761 2005-12-07 15:11:58Z niallp $
 * $Revision: 354761 $
 * $Date: 2005-12-07 10:11:58 -0500 (Wed, 07 Dec 2005) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2005 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.commons.resources.impl;

import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;

import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * <p>Unit tests for
 * <code>org.apache.commons.resources.impl.JDBCResources</code>.
 * </p>
 *
 * @author James Mitchell
 * @version $Revision: 354761 $ $Date: 2005-12-07 10:11:58 -0500 (Wed, 07 Dec 2005) $
 */

public class JDBCResourcesTestCase
    extends CollectionResourcesBaseTestCase {


    // ----------------------------------------------------- Instance Variables
    private Connection con;


    protected String getBase() throws Exception
    {
        URL url = 
            JDBCResourcesTestCase.class.getResource
            ("/org/apache/commons/resources/impl/jdbc.test.config.properties");
    
        if (url == null) {
            fail("URL NOT FOUND");
        }
        String string = url.toExternalForm();
        String base = string.substring(0, string.length() - 11);
        return base;
    }

    // ----------------------------------------------------------- Constructors


    public JDBCResourcesTestCase(String name) {
        super(name);
    }


    // --------------------------------------------------- Overall Test Methods


    // Set up instance variables required by this test case
    public void setUp() throws Exception {
        if (isPre14JVM()) {
            System.out.println("WARNING: CANNOT TEST with HSQLDB 1.7.2 ON PRE1.4 JVM");
            return;
        }

        factory = new JDBCResourcesFactory();
        resources = factory.getResources(NAME, getBase());
        
//        InputStream in = this.getClass().getResourceAsStream("init.sql");
//        File file = new File("init.sql");
        
        // change this to load from init.sql
        String[] sql = {""
        + "create table resources (" 
        + "  locale            varchar(10)      not null,"
        + "  msgKey            varchar(255)     not null,"
        + "  val               varchar(255),"
        + "  Primary Key("
        + "    locale,"
        + "    msgKey"
        + "  )"
        + " );"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.base', '[Base] ONLY');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.specific', '[Base] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.inherit', '[Base] INHERIT');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.message.single', '[Base] REPLACE {0}');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.message', '[Base] REPLACE {0} WITH {1}');"
        , "Insert into resources (locale, msgKey, val) Values ('en', 'test.specific', '[en] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('en', 'test.inherit', '[en] INHERIT');"
        , "Insert into resources (locale, msgKey, val) Values ('en_US', 'test.specific', '[en_US] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('fr', 'test.specific', '[fr] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('fr', 'test.inherit', '[fr] INHERIT');"
        };
        runSql(sql);
    }

    // Return the tests included in this test suite
    public static Test suite() {
        return (new TestSuite(JDBCResourcesTestCase.class));
    }

    // Tear down the instance variables required by this test case
    public void tearDown() {
        resources = null;
        factory = null;
        if (isPre14JVM()) {
            return;
        }
        try{
            runSql(new String[]{"drop table resources"});
            con.close();
        }catch(Exception e){
            // not really necessary to fail if creation also fails
            e.printStackTrace();
        }

    }
    public void testInherit() throws Exception {
        if (!isPre14JVM()) {
            super.testInherit();
        }
    }
    public void testLocaleList() {
        if (!isPre14JVM()) {
            super.testLocaleList();
        }
    }
    public void testMissing() throws Exception {
        if (!isPre14JVM()) {
            super.testMissing();
        }
    }
    public void testSpecific() throws Exception {
        if (!isPre14JVM()) {
            super.testSpecific();
        }
    }
    public void testKeys() throws Exception {
        if (!isPre14JVM()) {
            super.testKeys();
        }
    }
    public void testPristine() {
        if (!isPre14JVM()) {
            super.testPristine();
        }
    }
    public void testRelease() throws Exception {
        if (!isPre14JVM()) {
            super.testRelease();
        }
    }


    // ------------------------------------------------ Individual Test Methods


    // ------------------------------------------------------ Protected Methods


    // ------------------------------------------------------ Private Methods

    /**
     * @param sql
     * @throws SQLException
     */
    private void runSql(String[] sql) throws SQLException {

        String url = "jdbc:hsqldb:.";
        
        Statement stmt;
        try {
            Class.forName("org.hsqldb.jdbcDriver");

        } catch(java.lang.ClassNotFoundException e) {
            fail("ClassNotFoundException:" + e.getMessage());
        }

        try {
            // TODO change this to pull from jdbc.test.config.properties
            if (con == null || con.isClosed())
                con = DriverManager.getConnection(url, "sa", "");
            stmt = con.createStatement();
            
            for (int i = 0; i < sql.length; i++) {
                   stmt.execute(sql[i]);
            }
            stmt.close();
    
        } catch(SQLException ex) {
            fail("SQLException: " + ex.getMessage());
        }
    }

    private boolean isPre14JVM() {
        // some pre 1.4 JVM have buggy WeakHashMap implementations 
        // this is used to test for those JVM
        String version = System.getProperty("java.specification.version");
        StringTokenizer tokenizer = new StringTokenizer(version,".");
        if (tokenizer.nextToken().equals("1")) {
            String minorVersion = tokenizer.nextToken();
            if (minorVersion.equals("0")) return true;
            if (minorVersion.equals("1")) return true;
            if (minorVersion.equals("2")) return true;
            if (minorVersion.equals("3")) return true;
        }
        return false;
    }
}
